/*
 $License:
    Copyright (C) 2010 InvenSense Corporation, All Rights Reserved.
 $
 */
/******************************************************************************
 * $Id: eMPL-states.h $
 *****************************************************************************/
/**
 *  @defgroup   eMPL eMPL Library
 *  @brief      eMPL Layer.
 *  @details    The eMPL layer interfaces the user application with the MPL.
 *
 *  @{
 *      @file   eMPL-states.h
 *      @brief  Controls the eMPL state machine.
 */
#ifndef _EMPL_STATES_H_
#define _EMPL_STATES_H_

#include "mltypes.h"

/* Five power states:
 *  0.  EMPL_UNINIT        : Unititialized (only on boot)
 *  1.  EMPL_STOP          : All sensors off (suitable for host processor asleep)
 *  2.  EMPL_RUN           : MPU + Accel sensor fusion
 *  3.  EMPL_ACCEL_ONLY    : Accel only with motion interrupt
 *  4.  EMPL_LPACCEL_ONLY  : Accel only in Low power mode with motion interrupt
 *
 * Transitions:
 *  0 is the state on boot
 *  0       -> 1 : Call to eMPL_init
 *  1, 3, 4 -> 2 : Call to eMPL_start_mpu
 *  1, 2 ,4 -> 3 : Call to eMPL_start_accel_only
 *  1, 2, 3 -> 4 : Call to eMPL_start_lp_accel_only
 *  2, 3, 4 -> 1 : Call to eMPL_stop
 */

#define EMPL_UNINIT                     (0)
#define EMPL_STOP                       (1)
#define EMPL_RUN                        (2)
#define EMPL_ACCEL_ONLY                 (3)
#define EMPL_LPACCEL_ONLY               (4)

/**
 *  @brief  Get the current state of eMPL.
 *  @return eMPL state.
 */
int eMPL_get_state(void);

/**
 *  @brief	Entry point to eMPL.
 *  @e eMPL_init is the first function that should be called by the processor
 *  when using eMPL. It initializes the serial layer, and fetches calibration
 *  data if available.
 *  @return	INV_SUCCESS if successful.
 */
inv_error_t eMPL_init(void);

/**
 *  @brief		Starts the DMP and runs eMPL in full-power mode.
 *  This function also enables the default bias trackers.
 *  @param[in]	nine_axis	1 for 9-axis fusion, 0 for 6-axis fusion.
 *  @return		INV_SUCCESS if successful.
 */
inv_error_t eMPL_start_mpu(unsigned char nine_axis);

/**
 *  @brief  Stops the eMPL state machine and closes the DMP.
 *  @return INV_SUCCESS if successful.
 */
inv_error_t eMPL_stop(void);

/**
 *  @brief	Updates the motion library with fresh data from the sensor(s).
 *  @return	INV_SUCCESS if successful.
 */
inv_error_t eMPL_update_data(void);

#endif  /* _EMPL_STATES_H_ */

/**
 * @}
 */
