/*
 $License:
    Copyright (C) 2011 InvenSense Corporation, All Rights Reserved.
 $
 */
/******************************************************************************
 *
 * $Id:$
 *
 *****************************************************************************/

/**
 *  @defgroup eMPL eMPL Library
 *  @{
 *      @file ml_lite.h
 *      @brief Motion Library functions available in lite fusion.
 */

#ifndef INV_ML_LITE_H
#define INV_ML_LITE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "mltypes.h"
#define INV_MOTION          (0x0001)
#define INV_NO_MOTION       (0x0002)

#define INV_X_GYRO          (0x0001)
#define INV_Y_GYRO          (0x0002)
#define INV_Z_GYRO          (0x0004)
#define INV_DMP_PROCESSOR   (0x0008)

#define INV_X_ACCEL         (0x0010)
#define INV_Y_ACCEL         (0x0020)
#define INV_Z_ACCEL         (0x0040)

#define INV_X_COMPASS       (0x0080)
#define INV_Y_COMPASS       (0x0100)
#define INV_Z_COMPASS       (0x0200)

#define INV_TEMPERATURE     (0x2000)
#define INV_TIME            (0x4000)

#define INV_THREE_AXIS_GYRO (0x000F)
#define INV_THREE_AXIS_ACCEL    (0x0070)
#define INV_THREE_AXIS_COMPASS  (0x0380)

#define INV_FIVE_AXIS           (0x007B)
#define INV_SIX_AXIS_GYRO_ACCEL (0x007F)
#define INV_SIX_AXIS_ACCEL_COMPASS  (0x03F0)
#define INV_NINE_AXIS           (0x03FF)
#define INV_ALL_SENSORS         (0x7FFF)


	/**
	 *  @brief  inv_set_motion_callback is used to register a callback function that
	 *          will trigger when a change of motion state is detected.
	 *
	 *  @pre    inv_dmp_open()
	 *          and inv_dmp_start()
	 *          must <b>NOT</b> have been called.
	 *
	 *  @param  func    A user defined callback function accepting a
	 *                  lite_fusion->motion_state parameter, the new motion state.
	 *                  May be one of INV_MOTION or INV_NO_MOTION.
	 *  @return INV_SUCCESS if successful or Non-zero error code otherwise.
	 */
	inv_error_t
		inv_set_motion_callback(void (*func) (unsigned short motion_state));
	/**
	 * Controls each sensor and each axis when the motion processing unit is
	 * running.  When it is not running, simply records the state for later.
	 *
	 * NOTE: In this version only full sensors control is allowed.  Independent
	 * axis control will return an error.
	 *
	 * @param[in]	sensors Bit field of each axis desired to be turned on or off
	 *
	 * @return		INV_SUCCESS or non-zero error code
	 */
	inv_error_t inv_set_mpu_sensors(unsigned long sensors);
	/**
	 * @brief   Command the MPU to put data in the FIFO at a particular rate.
	 *
	 *          The DMP will add fifo entries every fifoRate + 1 MPU cycles.  For
	 *          example if the MPU is running at 200Hz the following values apply:
	 *
	 *          <TABLE>
	 *          <TR><TD>fifoRate</TD><TD>DMP Sample Rate</TD><TD>FIFO update frequency</TD></TR>
	 *          <TR><TD>0</TD><TD>200Hz</TD><TD>200Hz</TD></TR>
	 *          <TR><TD>1</TD><TD>200Hz</TD><TD>100Hz</TD></TR>
	 *          <TR><TD>2</TD><TD>200Hz</TD><TD>50Hz</TD></TR>
	 *          <TR><TD>4</TD><TD>200Hz</TD><TD>40Hz</TD></TR>
	 *          <TR><TD>9</TD><TD>200Hz</TD><TD>20Hz</TD></TR>
	 *          <TR><TD>19</TD><TD>200Hz</TD><TD>10Hz</TD></TR>
	 *          </TABLE>
	 *
	 *          Note: if the DMP is running, (state == INV_STATE_DMP_STARTED)
	 *          then inv_run_state_callbacks() will be called to allow features
	 *          that depend upon fundamental constants to be updated.
	 *
	 *  @pre    inv_dmp_open()
	 *          and inv_dmp_start()
	 *          must <b>NOT</b> have been called.
	 *
	 * @param   fifoRate    Divider value - 1.  Output rate is
	 *          (DMP Sample Rate) / (fifoRate + 1).
	 *
		 * @return  INV_SUCCESS if successful, ML error code on any failure.
		 */
	inv_error_t inv_set_fifo_rate(unsigned short fifoRate);
	/**
	 * @brief   Retrieve the current FIFO update divider - 1.
	 *          See inv_set_fifo_rate() for how this value is used.
	 *
	 *          The fifo rate when there is no fifo is the equivilent divider when
	 *          derived from the value set by SetSampleSteSizeMs()
	 *
	 * @return  The value of the fifo rate divider or INV_INVALID_FIFO_RATE on error.
	 */
	unsigned short inv_get_fifo_rate(void);
	/**
 	 *  @brief  Returns 3-element vector of gyro data in body frame.
 	 *  @param[out] data
 	 *              3-element vector of gyro data in body frame
 	 *              with gravity removed. One degree per second = 2^16.
 	 *  @return 0 on success or an error code.
 	 */
    inv_error_t inv_get_gyro(long *data);
    /**
     *  @brief       Returns 3-element vector of accelerometer data in body frame.
     *  @param[out]  data    3-element vector of accelerometer data in body frame.
     *                      One gee = 2^16.
     *  @return     0 on success or an error code.
     */
    inv_error_t inv_get_accel(long *data);
    inv_error_t inv_get_temperature(long *data);
    inv_error_t inv_get_temperature_raw(short *data);

    /**
     *  @brief  inv_get_rot_mat is used to get the rotation matrix
     *          representation of the current sensor fusion solution.
     *          The array format will be R11, R12, R13, R21, R22, R23, R31, R32,
     *          R33, representing the matrix:
     *          <center>R11 R12 R13</center>
     *          <center>R21 R22 R23</center>
     *          <center>R31 R32 R33</center>
     *          Values are scaled, where 1.0 = 2^30 LSBs.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 9 cells long</b>.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_rot_mat(long *data);
    /**
     *  @brief  inv_get_rot_vector is used to get the rotation vector
     *          representation of the current sensor fusion solution.
     *          Values are scaled, where 1.0 = 2^30 LSBs.
     *          A rotation vector is a method to represent a 4-element quaternion
     *          vector in 3-elements. To get the quaternion from the 3-elements,
     *          The last 3-elements of the quaternion will be the given rotation
     *          vector. The first element of the quaternion will be the positive
     *          value that will be required to make the magnitude of the quaternion
     *          1.0 or 2^30 in fixed point units.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_rot_vector(long *data);
    /**
     *  @brief      Returns 4-element quaternion vector derived from 6-axis or
     *  9-axis if 9-axis was implemented. 6-axis is gyros and accels. 9-axis is
     *  gyros, accel and compass.
     *
     *  @param[out] data    4-element quaternion vector. One is scaled to 2^30.
     *  @return     0 on success or an error code.
     */
    inv_error_t inv_get_quaternion(long *data);
    /**
     *  @brief      Returns 3-element vector of accelerometer data in body frame
     *              with gravity removed.
     *  @param[out] data    3-element vector of accelerometer data in body frame
     *                      with gravity removed. One g = 2^16.
     *  @return     0 on success or an error code. data unchanged on error.
     */
    inv_error_t inv_get_linear_accel(long *data);
    /**
     *  @brief      Returns 3-element vector of accelerometer data in world frame
     *              with gravity removed.
     *  @param[out] data    3-element vector of accelerometer data in world frame
     *                      with gravity removed. One g = 2^16.
     *  @return     0 on success or an error code.
     */
    inv_error_t inv_get_linear_accel_in_world(long *data);
    inv_error_t inv_get_gravity(long *data);

    /**
     *  @brief  inv_get_gyro_bias is used to get the gyroscope biases.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are scaled such that 1 dps = 2^16 LSBs.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_gyro_bias(long *data);
    /**
     *  @brief  inv_get_accel_bias is used to get the accelerometer baises.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are scaled such that 1 g (gravity) = 2^16 LSBs.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long </b>.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_accel_bias(long *data);
    /**
     *  @brief  inv_get_heading is used to get heading from Rotation Matrix.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to data to be passed back to the user.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_heading(long *data);
    /**
     *  @brief  inv_set_gyro_bias is used to set the gyroscope bias.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are scaled at 1 dps = 2^16 LSBs.
     *
     *          Please refer to the provided "9-Axis Sensor Fusion
     *          Application Note" document provided.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or
     *          MLDmpPedometerStandAloneOpen() \endif
     *
     *  @param  data        A pointer to an array to be copied from the user.
     *
     *  @return INV_SUCCESS if successful; a non-zero error code otherwise.
     */
    inv_error_t inv_set_gyro_bias(long *data);
    /**
     *  @brief  inv_set_accel_bias is used to set the accelerometer bias.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are scaled in units of g (gravity),
     *          where 1 g = 2^16 LSBs.
     *
     *          Please refer to the provided "9-Axis Sensor Fusion
     *          Application Note" document provided.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or
     *          MLDmpPedometerStandAloneOpen() \endif
     *
     *  @param  data        A pointer to an array to be copied from the user.
     *
     *  @return INV_SUCCESS if successful; a non-zero error code otherwise.
     */
    inv_error_t inv_set_accel_bias(long *data);
    /**
     *  @brief  inv_get_gyro_float is used to get the most recent gyroscope measurement.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are in units of dps (degrees per second).
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_gyro_float(float *data);
    /**
     *  @brief      Returns 3-element vector of accelerometer data in body frame.
     *  @param[out] data    3-element vector of accelerometer data in body frame in g's.
     *  @return     0 for success or an error code.
     */
    inv_error_t inv_get_accel_float(float *data);
    /**
     *  @brief  inv_get_temperature_float is used to get the most recent
     *          temperature measurement.
     *          The argument array should only have one element.
     *          The value is in units of deg C (degrees Celsius).
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to data to be passed back to the user.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_temperature_float(float *data);
    /**
     *  @brief  inv_get_rot_mat_float is used to get an array of nine data points representing the rotation
     *          matrix generated from all available sensors.
     *          The array format will be R11, R12, R13, R21, R22, R23, R31, R32,
     *          R33, representing the matrix:
     *          <center>R11 R12 R13</center>
     *          <center>R21 R22 R23</center>
     *          <center>R31 R32 R33</center>
     *          <b>Please refer to the "9-Axis Sensor Fusion Application Note" document,
     *          section 7 "Sensor Fusion Output", for details regarding rotation
     *          matrix output</b>.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 9 cells long at least</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_rot_mat_float(float *data);
    /**
     *  @brief  inv_get_rot_vector is used to get the rotation vector
     *          representation of the current sensor fusion solution.
     *          Values are scaled, where 1.0 = 2^30 LSBs.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_rot_vector_float(float *data);
    /**
     *  @brief      Returns 4-element quaternion vector.
     *  @param[out] data    4-element quaternion vector.
     *  @return     0 on success, an error code otherwise.
     */
    inv_error_t inv_get_quaternion_float(float *data);
    /**
     *  @brief  inv_get_linear_accel_float is used to get an estimate of linear
     *          acceleration, based on the most recent accelerometer measurement
     *          and sensor fusion solution.
     *          The values are in units of g (gravity).
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_linear_accel_float(float *data);
    /**
     *  @brief  inv_get_linear_accel_in_world_float is used to get an estimate of
     *          linear acceleration, in the world frame,  based on the most
     *          recent accelerometer measurement and sensor fusion solution.
     *          The values are in units of g (gravity).
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_linear_accel_in_world_float(float *data);
    /**
     *  @brief  inv_get_gravity_float is used to get an estimate of the body frame
     *          gravity vector, based on the most recent sensor fusion solution.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long at least</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_gravity_float(float *data);
    /**
     *  @brief  inv_get_gyro_bias_float is used to get the gyroscope biases.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are in units of dps (degrees per second).

     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_gyro_bias_float(float *data);
    /**
     *  @brief  inv_get_accel_bias_float is used to get the accelerometer baises.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are in units of g (gravity).
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_accel_bias_float(float *data);
    /**
     *  @brief  inv_get_heading_float is used to get single number representing the heading of the device
     *          relative to the Earth, in which 0 represents North, 90 degrees
     *          represents East, and so on.
     *          The heading is defined as the direction of the +Y axis if the Y
     *          axis is horizontal, and otherwise the direction of the -Z axis.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to the data to be passed back to the user.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_heading_float(float *data);
    /**
     *  @brief  inv_set_gyro_bias_float is used to set the gyroscope bias.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are in units of dps (degrees per second).
     *
     *          Please refer to the provided "9-Axis Sensor Fusion
     *          Application Note" document provided.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or
     *          MLDmpPedometerStandAloneOpen() \endif
     *  @pre    MLDmpStart() must <b>NOT</b> have been called.
     *
     *  @param  data        A pointer to an array to be copied from the user.
     *
     *  @return INV_SUCCESS if successful; a non-zero error code otherwise.
     */
    inv_error_t inv_set_gyro_bias_float(float *data);
    /**
     *  @brief  inv_set_accel_bias_float is used to set the accelerometer bias.
     *          The argument array elements are ordered X,Y,Z.
     *          The values are in units of g (gravity).
     *
     *          Please refer to the provided "9-Axis Sensor Fusion
     *          Application Note" document provided.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or
     *          MLDmpPedometerStandAloneOpen() \endif
     *  @pre    MLDmpStart() must <b>NOT</b> have been called.
     *
     *  @param  data        A pointer to an array to be copied from the user.
     *
     *  @return INV_SUCCESS if successful; a non-zero error code otherwise.
     */
    inv_error_t inv_set_accel_bias_float(float *data);
    /**
     *  @brief  inv_get_mag_bias is used to get Magnetometer Bias
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long </b>. Compass bias in mounting frame.
     *              1uT = 2^16.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_mag_bias(long *data);
    /**
     *  @brief  inv_get_mag_raw_data is used to get Raw magnetometer data.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long </b>.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_mag_raw_data(long *data);
    /**
     *  @brief  inv_get_magnetometer is used to get magnetometer data.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return Zero if the command is successful; an ML error code otherwise.
     */
    inv_error_t inv_get_magnetometer(long *data);
    /**
     *  @brief  inv_set_mag_bias is used to set Compass Bias
     *
     *          Please refer to the provided "9-Axis Sensor Fusion
     *          Application Note" document provided.
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or
     *          MLDmpPedometerStandAloneOpen() \endif
     *  @pre    MLDmpStart() must <b>NOT</b> have been called.
     *
     *  @param  data  Compass Bias in Compass Mounting Frame. 2^16 = 1uT
     *
     *  @return INV_SUCCESS if successful; a non-zero error code otherwise.
     */
    inv_error_t inv_set_mag_bias(long *data);
    /**
     *  @brief  inv_get_mag_bias_float is used to get an array of three data points representing
     *          the compass biases.
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_mag_bias_float(float *data);
    /**
     *  @brief  inv_get_mag_raw_data_float is used to get Raw magnetometer data
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_mag_raw_data_float(float *data);
    /**
     *  @brief  inv_get_magnetometer_float is used to get magnetometer data
     *
     *  @pre    MLDmpOpen() \ifnot eMPL or MLDmpPedometerStandAloneOpen() \endif
     *          must have been called.
     *
     *  @param  data
     *              A pointer to an array to be passed back to the user.
     *              <b>Must be 3 cells long</b>.
     *
     *  @return INV_SUCCESS if the command is successful; an error code otherwise.
     */
    inv_error_t inv_get_magnetometer_float(float *data);
    /**
     *  @brief  inv_set_mag_bias_float is used to set compass bias
     *
     *          Please refer to the provided "9-Axis Sensor Fusion
     *          Application Note" document provided.
     *
     *  @pre    MLDmpOpen()\ifnot eMPL or
     *          MLDmpPedometerStandAloneOpen() \endif
     *  @pre    MLDmpStart() must <b>NOT</b> have been called.
     *
     *  @param  data  Compass Bias in Compass Mounting Frame in units of uT.
     *
     *  @return INV_SUCCESS if successful; a non-zero error code otherwise.
     */
    inv_error_t inv_set_mag_bias_float(float *data);

#ifdef __cplusplus
}
#endif
#endif /* INV_ML_LITE_H */
/**
 * @}
 */
