/*
 $License:
    Copyright (C) 2011 InvenSense Corporation, All Rights Reserved.
 $
 */
/******************************************************************************
 *
 * $Id:$
 *
 *****************************************************************************/

/**
 *  @defgroup   eMPL
 *  @{
 *      @file       example.c
 *      @brief      Example test application for eMPL.
 *      @details    This application shows how to initialize and run eMPL, as
 *                  well as retrieve data and output it via UART.
 */

#include "msp430.h"
#include "msp430_sysclock.h"
#include "msp430_timer.h"
#include "msp430_i2c.h"
#include "msp430_nvram.h"
#include "msp430_uart.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "ml_lite.h"
#include "packet.h"
#include "ustore_manager.h"
#include "eMPL-states.h"
#include "ml_mputest.h"
#include "temp_comp.h"

#define SEND_DATA_MS    (1000)
#define ARRAY_SIZE(array) (sizeof(array)/sizeof((array)[0]))
#define SEND_GYRO       (0x0001)
#define SEND_ACCEL      (0x0002)
#define SEND_QUAT       (0x0004)
#define SEND_TEMP       (0x0008)
#define SEND_GYRO_BIAS  (0x0010)
#define SEND_TEMP_SLOPE (0x0020)
#define SEND_ACCEL_BIAS (0x0040)
#define SEND_MAG        (0x0080)
#define SEND_MAG_BIAS   (0x0100)
#define SEND_HEADING    (0x0200)
#define SEND_ROT        (0x0400)

struct example_s {
    volatile char uart_in[3];
    unsigned short fifo_ms;
    unsigned char cur_sensor;
    unsigned short data_send;
};

static struct example_s state;
static const unsigned short sensor_config[] = {
    INV_NINE_AXIS,
    INV_SIX_AXIS_GYRO_ACCEL,
    INV_THREE_AXIS_GYRO,
    INV_THREE_AXIS_ACCEL,
    INV_THREE_AXIS_COMPASS
};

static signed char set_up_msp430(void);
static int rx_cb(char data);
static void motion_cb(unsigned short state);
static signed char handle_input(void);
static signed char send_quat(void);
static void packetize_data(long *data, unsigned char num_elements,
    unsigned char q, unsigned char tag, unsigned char *packet);
static void send_data(void);

/**
 *  @brief      UART RX callback.
 *  @param[in]  data    Byte received.
 */
static int rx_cb(char data)
{
    if (state.uart_in[0] == 'm') {
        if (state.uart_in[1] == 'f')
            state.uart_in[2] = data;
        else if (data == 'f')
            state.uart_in[1] = 'f';
        else
            memset(&state.uart_in, 0, sizeof(state.uart_in));
    } else if (data == 'm')
        state.uart_in[0] = 'm';

    return 0;
}

/**
 *  @brief      Motion state change callback.
 *  This function toggles an LED connected to P1.1.
 *  @param[in]  state   New motion state.
 */
static void motion_cb(unsigned short state)
{
    if (state == INV_MOTION)
        P1OUT |= 0x08;
    else if (state == INV_NO_MOTION)
        P1OUT &= ~0x08;
}

/**
 *  @brief  Set up the MSP430 communication ports and timer.
 *  @return 0 if successful.
 */
static signed char set_up_msp430(void)
{
    if (msp430_configure_sysclock())
        return -1;
    if (msp430_clock_enable())
        return -1;
    if (msp430_i2c_enable())
        return -1;
    if (msp430_uart_enable())
        return -1;
    if (msp430_nvram_enable())
        return -1;
    /* Set P1.2 and P1.3 to output (for testing purposes only). */
    P1DIR |= 0x0C;
    P1OUT |= 0x0C;
    return 0;
}

/**
 *  @brief      Send out a data packet via UART.
 *  @param[in]  data            Data in q16 format.
 *  @param[in]  num_elements    Number of q16 values.
 *  @param[in]  q               Number of fractional bytes.
 *  @param[in]  tag             Character to represent packet ('a', 'g', etc.).
 *  @param[out] packet          Final packet.
 */
static void packetize_data(long *data, unsigned char num_elements,
    unsigned char q, unsigned char tag, unsigned char *packet) {
    unsigned char ii;

    for (ii = 0; ii < num_elements; ii++) {
        packet[ii*2] = (unsigned char) ((uint16_t)(data[ii] >> (24-q)) % 256);
        packet[ii*2+1] = (unsigned char) ((uint16_t)(data[ii] >> (16-q)) % 256);
    }
    packet[8] = tag;
}

/**
 *  @brief  Sends requested data via UART.
 *  eMPL_send_packet is a function in eMPL used to transmit arrays of 16-bit
 *  data over UART. The packet is formatted as follows:
 *  packet[0]    = $
 *  packet[1]    = 1 for a debug packet (string of text)
 *                 2 for a quaternion packet
 *                 3 for a data packet
 *  packet[2-12] = data
 *  packet[13]   = \r
 *  packet[14]   = \n
 */
static void send_data(void)
{
    long data[3];
    unsigned char out[10];

    if (state.data_send & SEND_GYRO) {
        if (!inv_get_gyro(data)) {
            packetize_data(data, 3, 5, 'g', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }

    if (state.data_send & SEND_GYRO_BIAS) {
        if (!inv_get_gyro_bias(data)) {
            packetize_data(data, 3, 5, 'b', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }

    if (state.data_send & SEND_ACCEL) {
        if (!inv_get_accel(data)) {
            packetize_data(data, 3, 12, 'a', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }

    if (state.data_send & SEND_ACCEL_BIAS) {
        if (!inv_get_accel_bias(data)) {
            packetize_data(data, 3, 12, 'x', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }

    if (state.data_send & SEND_TEMP) {
        if (!inv_get_temperature(data)) {
            packetize_data(data, 1, 8, 't', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }

    if (state.data_send & SEND_TEMP_SLOPE) {
        if (!inv_get_gyro_temp_slope(data)) {
            packetize_data(data, 3, 8, 's', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }

    if (state.data_send & SEND_MAG) {
        if (!inv_get_magnetometer(data)) {
            packetize_data(data, 3, 8, 'm', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }

    if (state.data_send & SEND_MAG_BIAS) {
        if (!inv_get_mag_bias(data)) {
            packetize_data(data, 3, 8, 'c', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }

    if (state.data_send & SEND_HEADING) {
        if (!inv_get_heading(data)) {
            packetize_data(data, 1, 0, 'h', out);
            eMPL_send_packet(PACKET_TYPE_DATA, out);
        }
    }
}

/**
 *  @brief  Handle requests from a device connected via UART.
 *  @return -1 if 'q' was pressed.
 */
static signed char handle_input(void)
{
    signed char result = 0;

    switch (state.uart_in[2]) {
    case 'f':
    case 'F':
        /* Change FIFO rate. */
        if (state.fifo_ms == 200) {         /* 5Hz */
            inv_set_fifo_rate(19);
            state.fifo_ms = 100;
        } else if (state.fifo_ms == 100) {  /* 10Hz */
            inv_set_fifo_rate(9);
            state.fifo_ms = 50;
        } else if (state.fifo_ms == 50) {   /* 20Hz */
            inv_set_fifo_rate(39);
            state.fifo_ms = 200;
        }
        break;
    case 's':
    case 'S':
        /* Change sensors. */
        state.cur_sensor = (state.cur_sensor+1) % ARRAY_SIZE(sensor_config);
        state.data_send = 0;
        /* Handle 9x->6x and 6x->9x transitions. */
        if (sensor_config[state.cur_sensor] == INV_NINE_AXIS) {
            eMPL_stop();
            eMPL_start_mpu(1);
            motion_cb(INV_MOTION);
        }
        else if (sensor_config[state.cur_sensor] == INV_SIX_AXIS_GYRO_ACCEL) {
            eMPL_stop();
            eMPL_start_mpu(0);
            motion_cb(INV_MOTION);
        } else
            inv_set_mpu_sensors(sensor_config[state.cur_sensor]);
        break;
    case 't':
    case 'T':
        /* Self test. */
        inv_self_test_calibration_run();
        eMPL_stop();
        eMPL_start_mpu(1);
        break;
    case 'a':
    case 'A':
        if (sensor_config[state.cur_sensor] & INV_THREE_AXIS_ACCEL)
            state.data_send ^= (SEND_ACCEL | SEND_ACCEL_BIAS);
        break;
    case 'g':
    case 'G':
        if (sensor_config[state.cur_sensor] & INV_THREE_AXIS_GYRO)
            state.data_send ^= (SEND_GYRO | SEND_GYRO_BIAS);
        break;
    case 'm':
    case 'M':
        if (sensor_config[state.cur_sensor] & INV_THREE_AXIS_COMPASS)
            state.data_send ^= (SEND_MAG | SEND_MAG_BIAS | SEND_HEADING);
        break;
    case 'z':
    case 'Z':
        /* Exit loop. */
        result = -1;
        break;
    default:
        break;
    }
    memset(&state.uart_in, 0, sizeof(state.uart_in));
    return result;
}

/**
 *  @brief  Send the current quaternion.
 *  Quaternions are transmitted in q14 fixed point.
 *  @return 0 if successful.
 */
static signed char send_quat(void)
{
    static unsigned char packet_count;
    long quat[4];
    unsigned char out[10];

    if (inv_get_quaternion(quat))
        return -1;

    /* Translate each element from q30 to q14. */
    out[0] = (unsigned char) (quat[0] >> 24);
    out[1] = (unsigned char) ((uint16_t)(quat[0] >> 16) % 256);
    out[2] = (unsigned char) (quat[1] >> 24);
    out[3] = (unsigned char) ((uint16_t)(quat[1] >> 16) % 256);
    out[4] = (unsigned char) (quat[2] >> 24);
    out[5] = (unsigned char) ((uint16_t)(quat[2] >> 16) % 256);
    out[6] = (unsigned char) (quat[3] >> 24);
    out[7] = (unsigned char) ((uint16_t)(quat[3] >> 16) % 256);
    out[8] = 'q';
    out[9] = packet_count++;
    if (eMPL_send_packet(PACKET_TYPE_QUATERNION, out))
        return -1;
    return 0;
}

int main(void)
{
    unsigned long timestamp, last_update, last_send;

    /* Disable watchdog timer. For most MSP430 applications, this step must
     * be executed first.
     */
    WDTCTL = WDTPW + WDTHOLD;

    /* Set up hardware. eMPL is packaged with some barebones drivers for the
     * MSP430 hardware peripherals.
     */
    if (set_up_msp430())
        /* On failure, trap CPU. */
        while (1);

    /* The UART hardware driver includes a callback mechanism when a byte is
     * received. Register a simple function to store this byte globally.
     */
    msp430_uart_register_rx_cb(rx_cb);

    /* Initial setup. */
    if (eMPL_init())
        while (1);

    /* Wait for byte from serial port. Remove this line if application
     * should automatically start on power up.
     */
    /* while (!state.uart_in); */
    memset(&state.uart_in, 0, sizeof(state.uart_in));
    state.cur_sensor = 0;
    state.data_send = 0;

    /* Begin normal operation at 20Hz FIFO rate.
     * 0 for 6-axis, 1 for 9-axis.
     */
    if (eMPL_start_mpu(1))
        while (1);
    last_update = 0;
    last_send = 0;
    timestamp = 0;
    motion_cb(INV_MOTION);
    /* This application uses simple scheduling based on a millisecond tick
     * count. This means no low power mode transitions, but you're welcome
     * to add that functionality yourself if you'd like.
     */
    state.fifo_ms = 50; /* 20Hz -> 50ms */
    /* eMPL has a callback function executed at every motion state
     * transition. Toggle an LED or do some other cool stuff.
     * NOTE: This needs to be registered after a call to eMPL_start_mpu().
     */
    inv_set_motion_callback(motion_cb);

    /* CPU won't ever exit from main completely. */
    while (1) {
        if (state.uart_in[2])
            if (handle_input())
                break;

        msp430_get_clock_count(&timestamp);
        /* Try to read FIFO more frequently than its update rate in case the
         * gyro PLL is a little fast. Change this ratio as needed.
         */
        if (timestamp < (last_update + (state.fifo_ms>>1)))
            continue;
        msp430_get_clock_count(&last_update);

        /* eMPL_update_data fetches FIFO data and executes all the sensor
         * fusion algorithms. The frequency of this call should match the
         * FIFO rate as closely as possible.
         */
        if (eMPL_update_data())
            continue;
        P1OUT ^= 0x04;

        /* At each FIFO period, the current quaternion is transmitted. */
        if (sensor_config[state.cur_sensor] == INV_NINE_AXIS ||
            sensor_config[state.cur_sensor] == INV_SIX_AXIS_GYRO_ACCEL)
            send_quat();

        if (timestamp < (last_send + SEND_DATA_MS))
            continue;
        msp430_get_clock_count(&last_send);

        if (state.data_send)
            send_data();
    }

    eMPL_stop();
    while(1);
}
