/*
 $License:
    Copyright (C) 2011 InvenSense Corporation, All Rights Reserved.
 $
 */
/******************************************************************************
 * $Id: msp430_nvram.h $
 *****************************************************************************/
/**
 *  @defgroup MSP430 System Layer
 *  @brief  MSP430 System Layer APIs.
 *          To interface with any platform, eMPL needs access to various
 *          system layer functions.
 *
 *  @{
 *      @file       msp430_nvram.h
 *      @brief      Functions to access flash storage on the MSP430.
 *      @details    This driver assumes that eMPL is running on either the
 *                  MSP430F5438A or the MSP430F5529. Other chips may have less
 *                  flash memory than what is required, while other MSP430s may
 *                  not support non-volatile storage at all. See the correct
 *                  device datasheet for more details.
 *
 *                  Another Note to Reader,
 *
 *                  Again, this driver is written specifically for eMPL. We
 *                  already know that eMPL won't need an entire segment of
 *                  storage and won't perform simultaneous flash read/writes.
 *                  Thus, there's no checks for boundary conditions or
 *                  multi-threading support.
 *
 *                  Sincerely,
 *                  A Happy Employee at Invensense
 */
#ifndef _MSP430_NVRAM_H_
#define _MSP430_NVRAM_H_

/* @e nvmem_addr_t represents an offset with respect to the beginning of the
 * flash segment allocated for eMPL's calibration load/store routine.
 */
typedef unsigned short nvmem_addr_t;

/**
 *  @brief      Enable read/write operations to flash.
 *
 *  This function simply points the memory pointer to a location in flash
 *  memory to be allocated for eMPL's calibration data.
 *
 *  @return     0 if successful.
 */
int msp430_nvram_enable(void);
/**
 *  @brief      Read from flash memory.
 *
 *  @param[in]  offset  Location of first byte in flash segment.
 *  @param[in]  length  Number of bytes.
 *  @param[out] data    Data read from flash.
 *  @return     0 if successful.
 */
int msp430_nvram_read(nvmem_addr_t offset, unsigned short length, char *data);
/**
 *  @brief      Write to flash memory.
 *
 *  @param[in]  offset  Location of first byte in flash segment.
 *  @param[in]  length  Number of bytes.
 *  @param[in]  data    Data read from flash.
 *  @return     0 if successful.
 */
int msp430_nvram_write(nvmem_addr_t offset, unsigned short length, char *data);
/**
 *  @brief  Clear the data stored by eMPL.
 *  @return 0 if successful.
 */
int msp430_nvram_clear(void);

#endif  /* _MSP430_NVRAM_H_ */

/**
 * @}
 */
