/*
 $License:
    Copyright (C) 2011 InvenSense Corporation, All Rights Reserved.
 $
 */
/******************************************************************************
 * $Id: msp430_uart.h $
 *****************************************************************************/

/**
 *  @defgroup MSP430 System Layer
 *  @brief  MSP430 System Layer APIs.
 *          To interface with any platform, eMPL needs access to various
 *          system layer functions.
 *
 *  @{
 *      @file       msp430_uart.h
 *      @brief      UART driver needed by eMPL to output log messages and
 *                  other data.
 *      @details    eMPL outputs data to another UART device connected to the
 *                  MSP430's TX/RX pins. Alternatively, these pins can be
 *                  connected to USB-UART interfacing hardware to allow the
 *                  MSP430 to communicate with a PC. This driver works for the
 *                  following MSP430s:
 *
 *                  MSP430F5438A
 *                  MSP430F5528
 *                  MSP430F5529
 *
 *                  Dear Reader,
 *
 *                  Similar to the I2C driver used in this project, this UART
 *                  driver is specifically written for eMPL. Therefore, only
 *                  the bare minimum functionality is provided, such that the
 *                  MSP430 can send information from eMPL to the serial port,
 *                  as well as receive commands as needed from the user.
 *                  Other features such as error detection and support for
 *                  multiprocessor communication won't be present here.
 *
 *                  Finally, this driver is not multi-thread safe. It doesn't
 *                  cache data to save memory.
 *
 *                  Thanks for your time,
 *                  A Happy Employee at Invensense
 */

#ifndef _MSP430_UART_H_
#define _MSP430_UART_H_

/**
 *  @brief      Set up the UART port.
 *  @return     0 if successful.
 */
int msp430_uart_enable(void);
/**
 *  @brief      Send a string via UART.
 *
 *  NOTE: This is a blocking function and will not relinquish control of the
 *  processor until all the bytes have been sent. Deal with it. =]
 *
 *  @param[in]  length  Number of bytes.
 *  @param[in]  data    Data to be written.
 *  @return		0 if successful.
 */
int msp430_uart_tx(char length, char *data);
/**
 *  @brief      Register a function to be called whenever there is a byte in
 *              the RX buffer.
 *
 *  This driver only supports one callback. If multiple processes need to be
 *  executed, let the callback function handle them.
 *
 *  The registered function must take in a single parameter: the byte received.
 *  I'm sure that was an obvious one.
 *
 *  @param[in]  func    Function to be registered.
 *  @return		0 if successful.
 */
int msp430_uart_register_rx_cb(int (*func)(char));


#endif  /* _MSP430_UART_H_ */
/**
 * @}
 */
